package javax.swing.plaf.basic;

import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.Shape;
import javax.swing.text.GlyphView;
import javax.swing.text.ViewFactory;
import javax.swing.event.DocumentEvent;
import javax.swing.text.CompositeView;
import javax.swing.SizeRequirements;
import java.awt.Container;
import javax.swing.text.ParagraphView;
import java.awt.Component;
import java.awt.Insets;
import javax.swing.text.BoxView;
import javax.swing.text.JTextComponent;
import javax.swing.text.WrappedPlainView;
import javax.swing.text.PlainView;
import javax.swing.JTextArea;
import javax.swing.text.View;
import javax.swing.text.Element;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;

public class BasicTextAreaUI extends BasicTextUI
{
    public static ComponentUI createUI(JComponent component) {
        return new BasicTextAreaUI();
    }
    
    public BasicTextAreaUI() {
    }
    
    @Override
    protected String getPropertyPrefix() {
        return "TextArea";
    }
    
    @Override
    protected void installDefaults() {
        super.installDefaults();
    }
    
    @Override
    protected void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        super.propertyChange(propertyChangeEvent);
        if (propertyChangeEvent.getPropertyName().equals("lineWrap") || propertyChangeEvent.getPropertyName().equals("wrapStyleWord") || propertyChangeEvent.getPropertyName().equals("tabSize")) {
            this.modelChanged();
        }
        else if ("editable".equals(propertyChangeEvent.getPropertyName())) {
            this.updateFocusTraversalKeys();
        }
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        return super.getPreferredSize(component);
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        return super.getMinimumSize(component);
    }
    
    @Override
    public View create(Element element) {
        Object property = element.getDocument().getProperty("i18n");
        if (property != null && property.equals(Boolean.TRUE)) {
            return this.createI18N(element);
        }
        JTextComponent component = this.getComponent();
        if ((component instanceof JTextArea)) {
            JTextArea textArea = (JTextArea)component;
            if (textArea.getLineWrap()) {
                return new WrappedPlainView(element, textArea.getWrapStyleWord());
            }
            return new PlainView(element);
        }
        else {
            return null;
        }
    }
    
    View createI18N(Element elem) {
        String name = elem.getName();
        if (name != null) {
            if (name.equals("content")) {
                return new PlainParagraph(elem);
            }
            if (name.equals("paragraph")) {
                return new BoxView(elem, 1);
            }
        }
        return null;
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        Object property = ((JTextComponent)c).getDocument().getProperty("i18n");
        Insets insets = c.getInsets();
        if (Boolean.TRUE.equals(property)) {
            View rootView = this.getRootView((JTextComponent)c);
            if (rootView.getViewCount() > 0) {
                height = height - insets.top - insets.bottom;
                int top = insets.top;
                int baseline = BasicHTML.getBaseline(rootView.getView(0), width - insets.left - insets.right, height);
                if (baseline < 0) {
                    return -1;
                }
                return top + baseline;
            }
            else {
                return -1;
            }
        }
        else {
            return insets.top + c.getFontMetrics(c.getFont()).getAscent();
        }
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        return Component.BaselineResizeBehavior.CONSTANT_ASCENT;
    }
    
    static class PlainParagraph extends ParagraphView
    {
        PlainParagraph(Element elem) {
            super(elem);
            (this.layoutPool = new LogicalView(elem)).setParent(this);
        }
        
        @Override
        public void setParent(View parent) {
            super.setParent(parent);
            if (parent != null) {
                this.setPropertiesFromAttributes();
            }
        }
        
        @Override
        protected void setPropertiesFromAttributes() {
            Container container = this.getContainer();
            if (container != null && !container.getComponentOrientation().isLeftToRight()) {
                this.setJustification(2);
            }
            else {
                this.setJustification(0);
            }
        }
        
        @Override
        public int getFlowSpan(int index) {
            Container container = this.getContainer();
            if (container instanceof JTextArea && !((JTextArea)container).getLineWrap()) {
                return Integer.MAX_VALUE;
            }
            return super.getFlowSpan(index);
        }
        
        @Override
        protected SizeRequirements calculateMinorAxisRequirements(int axis, SizeRequirements r) {
            SizeRequirements calculateMinorAxisRequirements = super.calculateMinorAxisRequirements(axis, r);
            Container container = this.getContainer();
            if ((container instanceof JTextArea)) {
                if (!((JTextArea)container).getLineWrap()) {
                    calculateMinorAxisRequirements.minimum = calculateMinorAxisRequirements.preferred;
                }
                else {
                    calculateMinorAxisRequirements.minimum = 0;
                    calculateMinorAxisRequirements.preferred = this.getWidth();
                    if (calculateMinorAxisRequirements.preferred == Integer.MAX_VALUE) {
                        calculateMinorAxisRequirements.preferred = 100;
                    }
                }
            }
            return calculateMinorAxisRequirements;
        }
        
        @Override
        public void setSize(float width, float height) {
            if ((int)width != this.getWidth()) {
                this.preferenceChanged(null, true, true);
            }
            super.setSize(width, height);
        }
        
        static class LogicalView extends CompositeView
        {
            LogicalView(Element elem) {
                super(elem);
            }
            
            @Override
            protected int getViewIndexAtPosition(int n) {
                Element element = this.getElement();
                if (element.getElementCount() > 0) {
                    return element.getElementIndex(n);
                }
                return 0;
            }
            
            @Override
            protected boolean updateChildren(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, ViewFactory viewFactory) {
                return false;
            }
            
            @Override
            protected void loadChildren(ViewFactory f) {
                Element element = this.getElement();
                if (element.getElementCount() > 0) {
                    super.loadChildren(f);
                }
                else {
                    this.append(new GlyphView(element));
                }
            }
            
            @Override
            public float getPreferredSpan(int n) {
                if (this.getViewCount() != 1) {
                    throw new Error("One child view is assumed.");
                }
                return this.getView(0).getPreferredSpan(n);
            }
            
            @Override
            protected void forwardUpdateToView(View v, DocumentEvent e, Shape a, ViewFactory f) {
                v.setParent(this);
                super.forwardUpdateToView(v, e, a, f);
            }
            
            @Override
            public void paint(Graphics graphics, Shape shape) {
            }
            
            @Override
            protected boolean isBefore(int n, int n2, Rectangle rectangle) {
                return false;
            }
            
            @Override
            protected boolean isAfter(int n, int n2, Rectangle rectangle) {
                return false;
            }
            
            @Override
            protected View getViewAtPoint(int n, int n2, Rectangle rectangle) {
                return null;
            }
            
            @Override
            protected void childAllocation(int n, Rectangle rectangle) {
            }
        }
    }
}
